/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac             

   class      : FindResultDlg
   date       : 13.2.98
   description: a dialog that displays the search result of the find
                date dialog
   modified   : 13.03.98 - dialog is no longer initially visible

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*; 
import java.awt.event.*;
import java.util.*;
import java.text.SimpleDateFormat;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class FindResultDlg extends Dialog
  implements ActionListener,ItemListener,AdjustmentListener {

  // instance variable declarations
  private Frame parent;
  private boolean canceled   = false;
  private boolean autoCancel = false;
  private Button okButton;
  private Button viewButton;
  private List list;
  private TextField startTimeText;
  private TextField requireTimeText;
  private Scrollbar startTimeSB;
  private Scrollbar requireTimeSB;
  private Vector resultVector;
  private SimpleDateFormat dateFormatter;
  private SimpleDateFormat timeFormatter;
  private Date start,end,curStart,curEnd,date;
  private int min,hour,curMin,curHour;
  private Calendar calendar;
  private CalendarEntry entry;

  // inner classes
  class FindResultDlgListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      setCanceled(true);
      setVisible(false);
      dispose();
    } // end windowClosing

    public void windowActivated(WindowEvent e) {
    } // end WindowActivated

    public void windowDeactivated(WindowEvent e) {
      if (autoCancel) {
        setCanceled(true);
        setVisible(false);
        dispose();
      } // end if
    } // end WindowDeactivated

  } // end FindResultDlgListener

  // costructors
  public FindResultDlg(Frame parent, Vector resultVector, int hour,
                         int min, CalendarEntry entry) {
    super(parent);
    this.parent       = parent;
    this.resultVector = resultVector;
    init(hour,min,entry);
  } // end constructor FindResultDlg

  public FindResultDlg(Frame parent, boolean modal, Vector resultVector,
                         int hour, int min, CalendarEntry entry) {
    super(parent,modal);
    this.parent       = parent;
    this.resultVector = resultVector;
    init(hour,min,entry);
  } // end constructor FindResultDlg

  public FindResultDlg(Frame parent, String title, Vector resultVector,
                         int hour, int min, CalendarEntry entry) {
    super(parent,title);
    this.parent       = parent;
    this.resultVector = resultVector;
    init(hour,min,entry);
  } // end constructor FindResultDlg

  public FindResultDlg(Frame parent, String title, boolean modal,
                         Vector resultVector, int hour, int min,
                         CalendarEntry entry) {
    super(parent,title,modal);
    this.parent       = parent;
    this.resultVector = resultVector;
    init(hour,min,entry);
  } // end constructor FindResultDlg

  // methods
  private void init(int hour, int min, CalendarEntry entry) {
    this.hour  = hour;
    this.min   = min;
    this.entry = entry;
    addWindowListener(new FindResultDlgListener());
    // this dialog box uses the gridbaglayout to provide a pretty good layout
    GridBagLayout gridbag          = new GridBagLayout(); 
    GridBagConstraints constraints = new GridBagConstraints();

    // create the widgets
    InsetsPanel pan = new InsetsPanel(new Insets(15,20,15,20));
    okButton = new Button("Add");
    okButton.addActionListener(this);
    viewButton = new Button("Cancel");
    viewButton.addActionListener(this);
    InsetsPanel buttonPanel = new InsetsPanel(new Insets(15,0,0,0));
    buttonPanel.setLayout(new GridLayout(1,2,10,0));
    buttonPanel.add(okButton);
    buttonPanel.add(viewButton);

    dateFormatter = new SimpleDateFormat("d.MM.yy H:mm",
                           CalendarCreator.getLocale());
    timeFormatter = new SimpleDateFormat("H:mm",
                           CalendarCreator.getLocale());
    start         = new Date();
    end           = new Date();
    curStart      = new Date();
    curEnd        = new Date();
    date          = new Date();
    calendar      = CalendarCreator.get();

    Label label = new Label("The following periods of time are available"+
                            " for a date:");

    Label startTimeLabel   = new Label("Start Time:");
    Label requireTimeLabel = new Label("Time Require:");
    // group the time labels together
    Panel timeLabelPanel = new Panel(new GridLayout(2,1));
    timeLabelPanel.add(startTimeLabel);
    timeLabelPanel.add(requireTimeLabel);

    startTimeText   = new TextField("",14);
    startTimeText.setEditable(false);
    requireTimeText = new TextField("",5);
    requireTimeText.setEditable(false);
    startTimeSB     = new Scrollbar(Scrollbar.HORIZONTAL);
    startTimeSB.setBlockIncrement(60);
    startTimeSB.setUnitIncrement(1);
    startTimeSB.setMinimum(0);
    startTimeSB.addAdjustmentListener(this);
    requireTimeSB   = new Scrollbar(Scrollbar.HORIZONTAL);
    requireTimeSB.setBlockIncrement(60);
    requireTimeSB.setUnitIncrement(1);
    requireTimeSB.setMinimum(0);
    requireTimeSB.addAdjustmentListener(this);
    // group the time texts and scroll bars together
    Panel timeTextPanel = new Panel(new GridLayout(2,2));
    timeTextPanel.add(startTimeText);
    timeTextPanel.add(startTimeSB);
    timeTextPanel.add(requireTimeText);
    timeTextPanel.add(requireTimeSB);

    // set up the list
    {
      int n = resultVector.size();
      int i;
      String str;
      DateDupel dateDupel;

      list = new List(5,false);
      for (i=0; i<n; i++) {
        dateDupel = (DateDupel)resultVector.elementAt(i);
        str = "Start: "+dateFormatter.format(dateDupel.start);
        str += " End: "+dateFormatter.format(dateDupel.end);
        list.add(str);
      } // end for
      list.select(0);
      itemStateChanged(null);
      list.addItemListener(this);
    }

    pan.setLayout(gridbag);

    constraints.insets     = new Insets(5,6,5,6);
    constraints.fill       = GridBagConstraints.NONE; 
    constraints.anchor     = GridBagConstraints.WEST; 
    constraints.weightx    = 1.0; 
    constraints.weighty    = 1.0; 
    constraints.gridwidth  = GridBagConstraints.REMAINDER; 
    constraints.gridheight = 1; 
    gridbag.setConstraints(label,constraints); 
    pan.add(label); 

    constraints.anchor     = GridBagConstraints.CENTER; 
    gridbag.setConstraints(list,constraints); 
    pan.add(list); 

    SeparatorCanvas separator = new SeparatorCanvas();
    constraints.fill       = GridBagConstraints.HORIZONTAL; 
    gridbag.setConstraints(separator,constraints); 
    pan.add(separator);

    constraints.fill       = GridBagConstraints.NONE;
    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    gridbag.setConstraints(timeLabelPanel,constraints); 
    pan.add(timeLabelPanel);

    constraints.gridwidth  = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(timeTextPanel,constraints); 
    pan.add(timeTextPanel);

    separator = new SeparatorCanvas();
    constraints.fill       = GridBagConstraints.HORIZONTAL; 
    constraints.gridheight = GridBagConstraints.RELATIVE;
    gridbag.setConstraints(separator,constraints); 
    pan.add(separator);

    constraints.fill       = GridBagConstraints.NONE; 
    constraints.gridheight = GridBagConstraints.REMAINDER; 
    gridbag.setConstraints(buttonPanel, constraints); 
    pan.add(buttonPanel); 

    add(pan);
    pack(); 
    setResizable(false);
    //setVisible(true);
  } // end init

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("Cancel")) {
      setCanceled(true);
      setVisible(false);
      dispose();
    }
    else { // if cmd equals Add
      setVisible(false);
      dispose();
    } // end if
  } // end actionPerformed

  private void updateTexts(boolean onlySecond) {
    //int m,h;

    //calendar.setTime(curStart);
    //m = calendar.get(Calendar.MINUTE);
    //h = calendar.get(Calendar.HOUR_OF_DAY);
    //calendar.add(Calendar.MINUTE,-m+curMin);
    //calendar.add(Calendar.HOUR_OF_DAY,-h+curHour);
    //date.setTime(calendar.getTime().getTime());
    //requireTimeText.setText(timeFormatter.format(date));
    if (curMin < 10)
      requireTimeText.setText(curHour+":0"+curMin);
    else
      requireTimeText.setText(curHour+":"+curMin);
    // end if
    if (onlySecond) return;
    startTimeText.setText(dateFormatter.format(curStart));
  } // end updateTexts

  private void updateStartSB() {
    long cs = curStart.getTime()/60000;
    long s  = start.getTime()/60000;
    long e  = end.getTime()/60000;
    long d;

    startTimeSB.setMaximum((int)(e-s));
    calendar.setTime(start);
    calendar.add(Calendar.MINUTE,curMin);
    calendar.add(Calendar.HOUR_OF_DAY,curHour);
    d = calendar.getTime().getTime()/60000;
    startTimeSB.setVisibleAmount((int)(d-s));
    startTimeSB.setValue((int)(cs-s));
  } // end updateStartSB

  private void updateRequireSB() {
    long cs = curStart.getTime()/60000;
    long s  = start.getTime()/60000;
    long e  = end.getTime()/60000;
    long d;
    int i = (int)(e-s);

    requireTimeSB.setMaximum(i > 24*60 ? 24*60 : i);
    calendar.setTime(start);
    calendar.add(Calendar.MINUTE,curMin);
    calendar.add(Calendar.HOUR_OF_DAY,curHour);
    d = calendar.getTime().getTime()/60000;
    i = (int)(cs-s+1);
    requireTimeSB.setVisibleAmount(i > 24*60 ? 24*60 : i);
    requireTimeSB.setValue((int)(d-s-1));
  } // end updateRequireSB

  public void itemStateChanged(ItemEvent e) {
    int index           = list.getSelectedIndex();
    DateDupel dateDupel = (DateDupel)resultVector.elementAt(index);

    start.setTime(dateDupel.start.getTime());
    end.setTime(dateDupel.end.getTime());
    curStart.setTime(start.getTime());
    curEnd.setTime(end.getTime());
    curMin  = min;
    curHour = hour;
    updateTexts(false);
    updateStartSB();
    updateRequireSB();
  } // end itemStateChanged

  public void adjustmentValueChanged(AdjustmentEvent e) {
    boolean secondSB = (requireTimeSB == (Scrollbar)e.getSource());
    long d;
    int i;

    if (secondSB) {
      i       = requireTimeSB.getValue()+1;
      curHour = i / 60;
      curMin  = i % 60;
      startTimeSB.setVisibleAmount(i);
      updateTexts(true);
    }
    else {
      curStart.setTime(60000*(long)startTimeSB.getValue()+start.getTime());
      //d = (curStart.getTime()-start.getTime())/60000;
      d = (curStart.getTime()-end.getTime())/60000+requireTimeSB.getMaximum();
      i = d < 0 ? 1 : (int)(d+1);
      //requireTimeSB.setVisibleAmount(i > 24*60 ? 24*60 : i);
      requireTimeSB.setVisibleAmount(i);
      updateTexts(false);
    } // end if
  } // end adjustmentValueChanged

  public CalendarEntry getCalendarEntry() {
    entry.setStart(curStart);
    calendar.setTime(curStart);
    calendar.add(Calendar.MINUTE,curMin);
    calendar.add(Calendar.HOUR_OF_DAY,curHour);
    entry.setEnd(calendar.getTime());
    return entry;
  } // end getCalendarEntry

  public boolean getCanceled() {
    return canceled;
  } // end getCanceled

  public void setCanceled(boolean canceled) {
    this.canceled = canceled;
  } // end getCanceled

  public boolean getAutoCancel() {
    return autoCancel;
  } // end getAutoCancel

  public void setAutoCancel(boolean autoCancel) {
    this.autoCancel = autoCancel;
  } // end getAutoCancel

} // end FindResultDlg

/* ******************************************************************
   end of file
*********************************************************************/
