/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac             

   class      : SysInfoDlg
   date       : 7.12.97
   description: a dialog that displays system informations
   modified   : 3.2.98 - auto cancel mode for applet operation added

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/


import java.awt.*; 
import java.awt.event.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class SysInfoDlg extends Dialog implements ActionListener {

  // class variable declarations
  private final static String[] properties = {
    "java.version",
    "java.vendor",
    "java.vendor.url",
    "java.class.version",
    "java.compiler",
    "os.name",
    "os.arch",
    "os.version",
    "file.separator",
    "path.separator"
  };

  // instance variable declarations
  private boolean autoCancel = false;
  private Button okButton;
  private Button gcButton;
  private Label percentLabel;
  private int freePercent;

  // inner classes
  class SysInfoDlgListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      setVisible(false);
      dispose();
    } // end windowClosing

    public void windowDeactivated(WindowEvent e) {
      if (autoCancel) {
        setVisible(false);
        dispose();
      } // end if
    } // end WindowDeactivated

  } // end SysInfoDlgListener

  // costructors
  public SysInfoDlg(Frame parent) {
    super(parent);
    init();
  } // end constructor SysInfoDlg

  public SysInfoDlg(Frame parent, boolean modal) {
    super(parent,modal);
    init();
  } // end constructor SysInfoDlg

  public SysInfoDlg(Frame parent, String title) {
    super(parent,title);
    init();
  } // end constructor SysInfoDlg

  public SysInfoDlg(Frame parent, String title, boolean modal) {
    super(parent,title,modal);
    init();
  } // end constructor SysInfoDlg

  // methods
  private void init() {
    int n,i;
    String str;

    addWindowListener(new SysInfoDlgListener());
    // this dialog box uses the gridbaglayout to provide a pretty good layout
    GridBagLayout gridbag          = new GridBagLayout(); 
    GridBagConstraints constraints = new GridBagConstraints();

    // create the widgets
    InsetsPanel pan = new InsetsPanel(new Insets(15,20,15,20));
    okButton = new Button("OK");
    okButton.addActionListener(this);
    gcButton = new Button("Collect");
    gcButton.addActionListener(this);

    InsetsPanel buttonPanel = new InsetsPanel(new Insets(15,0,0,0));
    buttonPanel.setLayout(new GridLayout(1,2,10,0));
    buttonPanel.add(okButton);
    buttonPanel.add(gcButton);

    n = properties.length;
    Panel labelPanel = new Panel();
    labelPanel.setLayout(new GridLayout(n+2,2));
    for (i=0; i<n; i++) {
      labelPanel.add(new Label(properties[i]));
      str = System.getProperty(properties[i]);
      if (str == null) str = "Not specified";
      labelPanel.add(new Label(str));
    } // end for
    Runtime r = Runtime.getRuntime();
    labelPanel.add(new Label("total.memory"));
    labelPanel.add(new Label(r.totalMemory()+" Bytes"));
    labelPanel.add(new Label("free.memory"));
    updateFreePercent(r);
    percentLabel = new Label(freePercent+" %    ");
    labelPanel.add(percentLabel);
    pan.setLayout(gridbag); 

    constraints.fill       = GridBagConstraints.NONE; 
    constraints.anchor     = GridBagConstraints.CENTER; 
    constraints.weightx    = 1.0; 
    constraints.weighty    = 1.0; 
    constraints.gridwidth  = GridBagConstraints.REMAINDER; 
    constraints.gridheight = GridBagConstraints.RELATIVE; 
    gridbag.setConstraints(labelPanel,constraints); 
    pan.add(labelPanel); 

    constraints.ipadx      = 0; 
    constraints.ipady      = 0; 
    constraints.weightx    = 0.0; 
    constraints.weighty    = 0.0; 
    constraints.gridwidth  = 1; 
    constraints.gridheight = GridBagConstraints.REMAINDER; 

    gridbag.setConstraints(buttonPanel, constraints); 
    pan.add(buttonPanel); 

    add(pan);
    pack(); 
    setVisible(true);
  } // end init

  private void updateFreePercent(Runtime r) {
    if (r == null) r = Runtime.getRuntime();
    freePercent = (int)(100 * r.freeMemory() / r.totalMemory());
  } // end updatePercent

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("Collect")) {
      System.gc();
      updateFreePercent(null);
      percentLabel.setText(freePercent+" %    ");
    }
    else { // if cmd equals ok
      setVisible(false);
      dispose();
    } // end if
  } // end actionPerformed

  public boolean getAutoCancel() {
    return autoCancel;
  } // end getAutoCancel

  public void setAutoCancel(boolean autoCancel) {
    this.autoCancel = autoCancel;
  } // end getAutoCancel

} // end SysInfoDlg

/* ******************************************************************
   end of file
*********************************************************************/
